\ DX-Forth library for creating strings by appending
\ Memory space or length of strings (max. 255) are NOT checked!
\ Pavel Zampach, 2020
\ *

cr .( Append string routines loading )

\ defining word for string constant
\ definition: cstring name Some text"
system
: CSTRING     ( "name" ccc" -- ; runtime:  -- addr u )
  ['] count build ,"
;

\ defining word for string variable
\ definition: len vstring name
: VSTRING     ( n "name" -- ; runtime:  -- addr u )
  ['] count build 0 C, allot
;
application

\ reverse to COUNT
: -COUNT      ( addr1 u -- addr2)
  drop 1-
;

\ place string [addr2 u2] to string variable [addr1 u1]
: APPOKE      ( addr1 u1 addr2 u2 -- )
  2swap -count place
;

\ init for appending to string variable [addr u]
: <APP        ( addr u -- addr 0 )
  drop 0
;

\ init for appending to address [addr]
: <ADR        ( addr -- addr+1 0 )
  1+ 0
;

: <PAD        ( -- addr 0 )
  pad <adr
;

\ close appending, save string length to the first byte
: APP>        ( addr u -- )
  swap 1- c!
;

\ append char [c] to string [addr u]
: APPC        ( addr u c -- addr u+1 )
  >r 2dup + r> swap c! 1+
;

\ append [n] spaces to string [addr u]
: APPB        ( addr u n -- addr u+n )
  0 ?do bl appc loop
;

\ append string [addr2 u2] to string [addr1 u1]
: APPS        ( addr1 u1 addr2 u2 -- addr1 u1+u2 )
  2swap +string
;

\ append number [n] to string [addr u1]
: APPN        ( addr u1 n -- addr u2 )
  (.) apps
;

\ append string [addr2 u2] to string [addr1 u1] padding right in [n] field
: APPSR       ( add1 u1 addr2 u2 n -- -- addr1 u1+u2+n )
  over - 0 max >r
  2swap r> appb +string
;

\ append string [addr2 u2] to string [addr1 u1] padding left in [n] field
: APPSL       ( add1 u1 addr2 u2 n -- -- addr1 u1+u2+n )
  over - 0 max >r
  apps r> appb
;
