\ DX-Forth library for access to dBase II DBF files
\ Memory space or length of strings are NOT checked!
\ Pavel Zampach, 2020
\ *

cr .( Access to dBASE II files )
\ DBFCONST.F must be included before!

create RECBUFF                          \ record buffer
  recsize allot
0 value DBFID
0 value RECS#                           \ No of records

: FILE-CHECK  ( ior -- )
  abort" File access error"
;

\ Open DBF filename [addr u], extension isn't needed
: OPEN-DBF    ( addr u -- )
  s" DBF" +ext
  r/w open-file file-check to dbfid
  recbuff 3 dbfid read-file file-check drop
  recbuff c@ 2 <> abort" No dBase II file"
  recbuff 1+ @ to recs#
;

\ Close DBF file
: CLOSE-DBF   ( -- )
  dbfid close-file file-check
;

\ Set position [n] in DBF file
: SET-POSITION ( n -- )
  dup 1+ recs# > abort" Index too large"
  recsize m* 521. d+
  dbfid reposition-file file-check
;

\ Read record number [n] to record buffer
\ record numbers begin with zero!
: READ-REC    ( n -- )
  set-position
  recbuff recsize dbfid read-file file-check drop
;

\ Write record number [n] from record buffer
\ record numbers begin with zero!
: WRITE-REC   ( n -- )
  set-position
  recbuff recsize dbfid write-file file-check
;

\ True if record is "alive" (not marked as deleted)
: GET-ALIVE   ( -- flag )
  recbuff c@ bl =
;

\ Set record as "alive" (not marked as deleted)
: SET-ALIVE   ( flag -- )
  if bl else [char] * then recbuff c!
;

\ Get string [addr u] from field name set in DBFCONST
: GET-FIELDC  ( len off -- addr u )
  recbuff + swap
;

\ Get number [n] from field name set in DBFCONST
: GET-FIELDN  ( len off -- n|0 )
  get-fieldc bl skip
  number? if d>s else 0 then
;

\ Get logical value from field name set in DBFCONST
: GET-FIELDL  ( len off -- flag )
  nip recbuff + c@ upcase
  dup [char] Y =
  over [char] T = or
  swap [char] 1 = or
;

\ Blank field given by field name set in DBFCONST
\ and return field as string [addr u]
: BLANK-FIELD ( len off -- addr u )
  get-fieldc 2dup bl fill
;

\ Truncate string [addr1 u1] to fit into field [addr2 u2]
: TRIM-LEN    ( addr1 u1 addr2 u2 -- addr1 u3 addr2 u2)
  rot over min -rot
;

\ Set string [addr u] padding right to field name set in DBFCONST
: SET-FIELDCR ( addr u len off -- )
  blank-field trim-len
  + over - swap cmove
;

\ Set string [addr u] padding left to field name set in DBFCONST
: SET-FIELDCL ( addr u len off -- )
  blank-field trim-len
  drop swap cmove
;

\ Set number [n] to field name set in DBFCONST
: SET-FIELDN  ( n len off -- )
  rot (.) 2swap set-fieldcr
;

\ Set logical value to field name set in DBFCONST
: SET-FIELDL  ( flag len off -- )
  nip recbuff + swap
  if [char] Y else [char] N then
  swap c!
;
